@@ -1,6 +1,7 @@
-/* $Id: thunar-location-button.c 28973 2008-12-02 18:23:11Z nick $ */
+/* $Id$ */
 /*-
  * Copyright (c) 2005-2006 Benedikt Meurer <benny@xfce.org>
+ * Copyright (c) 2009 Jannis Pohlmann <jannis@xfce.org>
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the Free
@@ -30,9 +31,11 @@
 
 #include <thunar/thunar-application.h>
 #include <thunar/thunar-dnd.h>
+#include <thunar/thunar-gio-extensions.h>
 #include <thunar/thunar-icon-factory.h>
 #include <thunar/thunar-location-button.h>
 #include <thunar/thunar-pango-extensions.h>
+#include <thunar/thunar-preferences.h>
 #include <thunar/thunar-private.h>
 
 
@@ -56,8 +59,6 @@ enum
 
 
 
-static void           thunar_location_button_class_init             (ThunarLocationButtonClass  *klass);
-static void           thunar_location_button_init                   (ThunarLocationButton       *location_button);
 static void           thunar_location_button_finalize               (GObject                    *object);
 static void           thunar_location_button_get_property           (GObject                    *object,
                                                                      guint                       prop_id,
@@ -72,7 +73,7 @@ static void           thunar_location_button_style_set              (GtkWidget
 static GdkDragAction  thunar_location_button_get_dest_actions       (ThunarLocationButton       *location_button,
                                                                      GdkDragContext             *context,
                                                                      GtkWidget                  *button,
-                                                                     guint                       time);
+                                                                     guint                       timestamp);
 static void           thunar_location_button_file_changed           (ThunarLocationButton       *location_button,
                                                                      ThunarFile                 *file);
 static void           thunar_location_button_file_destroy           (ThunarLocationButton       *location_button,
@@ -90,13 +91,13 @@ static gboolean       thunar_location_button_drag_drop              (GtkWidget
                                                                      GdkDragContext             *context,
                                                                      gint                        x,
                                                                      gint                        y,
-                                                                     guint                       time,
+                                                                     guint                       timestamp,
                                                                      ThunarLocationButton       *location_button);
 static void           thunar_location_button_drag_data_get          (GtkWidget                  *button,
                                                                      GdkDragContext             *context,
                                                                      GtkSelectionData           *selection_data,
                                                                      guint                       info,
-                                                                     guint                       time,
+                                                                     guint                       timestamp,
                                                                      ThunarLocationButton       *location_button);
 static void           thunar_location_button_drag_data_received     (GtkWidget                  *button,
                                                                      GdkDragContext             *context,
@@ -104,17 +105,17 @@ static void           thunar_location_button_drag_data_received     (GtkWidget
                                                                      gint                        y,
                                                                      GtkSelectionData           *selection_data,
                                                                      guint                       info,
-                                                                     guint                       time,
+                                                                     guint                       timestamp,
                                                                      ThunarLocationButton       *location_button);
 static void           thunar_location_button_drag_leave             (GtkWidget                  *button,
                                                                      GdkDragContext             *context,
-                                                                     guint                       time,
+                                                                     guint                       timestamp,
                                                                      ThunarLocationButton       *location_button);
 static gboolean       thunar_location_button_drag_motion            (GtkWidget                  *button,
                                                                      GdkDragContext             *context,
                                                                      gint                        x,
                                                                      gint                        y,
-                                                                     guint                       time,
+                                                                     guint                       timestamp,
                                                                      ThunarLocationButton       *location_button);
 static gboolean       thunar_location_button_enter_timeout          (gpointer                    user_data);
 static void           thunar_location_button_enter_timeout_destroy  (gpointer                    user_data);
@@ -137,10 +138,10 @@ struct _ThunarLocationButton
   ThunarFileIconState file_icon_state;
   
   /* enter folders using DnD */
-  gint                enter_timeout_id;
+  guint               enter_timeout_id;
 
   /* drop support for the button */
-  GList              *drop_path_list;
+  GList              *drop_file_list;
   guint               drop_data_ready : 1;
   guint               drop_occurred : 1;
 
@@ -156,37 +157,11 @@ static const GtkTargetEntry drag_targets[] =
   { "text/uri-list", 0, 0 },
 };
 
-static GObjectClass *thunar_location_button_parent_class;
-static guint         location_button_signals[LAST_SIGNAL];
+static guint location_button_signals[LAST_SIGNAL];
 
 
 
-GType
-thunar_location_button_get_type (void)
-{
-  static GType type = G_TYPE_INVALID;
-
-  if (G_UNLIKELY (type == G_TYPE_INVALID))
-    {
-      static const GTypeInfo info =
-      {
-        sizeof (ThunarLocationButtonClass),
-        NULL,
-        NULL,
-        (GClassInitFunc) thunar_location_button_class_init,
-        NULL,
-        NULL,
-        sizeof (ThunarLocationButton),
-        0,
-        (GInstanceInitFunc) thunar_location_button_init,
-        NULL,
-      };
-
-      type = g_type_register_static (GTK_TYPE_ALIGNMENT, I_("ThunarLocationButton"), &info, 0);
-    }
-
-  return type;
-}
+G_DEFINE_TYPE (ThunarLocationButton, thunar_location_button, GTK_TYPE_ALIGNMENT)
 
 
 
@@ -196,9 +171,6 @@ thunar_location_button_class_init (ThunarLocationButtonClass *klass)
   GtkWidgetClass *gtkwidget_class;
   GObjectClass   *gobject_class;
 
-  /* determine the parent type class */
-  thunar_location_button_parent_class = g_type_class_peek_parent (klass);
-
   gobject_class = G_OBJECT_CLASS (klass);
   gobject_class->finalize = thunar_location_button_finalize;
   gobject_class->get_property = thunar_location_button_get_property;
@@ -246,8 +218,9 @@ thunar_location_button_class_init (ThunarLocationButtonClass *klass)
                   G_TYPE_FROM_CLASS (klass),
                   G_SIGNAL_RUN_LAST,
                   0, NULL, NULL,
-                  g_cclosure_marshal_VOID__VOID,
-                  G_TYPE_NONE, 0);
+                  g_cclosure_marshal_VOID__BOOLEAN,
+                  G_TYPE_NONE, 1,
+                  G_TYPE_BOOLEAN);
 
   /**
    * ThunarLocationButton::context-menu:
@@ -286,13 +259,13 @@ thunar_location_button_class_init (ThunarLocationButtonClass *klass)
 static void
 thunar_location_button_init (ThunarLocationButton *location_button)
 {
-  GtkWidget *button;
   GtkWidget *align;
+  GtkWidget *button;
   GtkWidget *hbox;
 
   /* create the toggle button */
   button = gtk_toggle_button_new ();
-  GTK_WIDGET_UNSET_FLAGS (button, GTK_CAN_FOCUS);
+  gtk_widget_set_can_focus (button, FALSE);
   g_signal_connect_swapped (G_OBJECT (button), "clicked", G_CALLBACK (thunar_location_button_clicked), location_button);
   exo_mutual_binding_new (G_OBJECT (location_button), "active", G_OBJECT (button), "active");
   gtk_container_add (GTK_CONTAINER (location_button), button);
@@ -340,7 +313,7 @@ thunar_location_button_finalize (GObject *object)
   ThunarLocationButton *location_button = THUNAR_LOCATION_BUTTON (object);
 
   /* release the drop path list (just in case the drag-leave wasn't fired before) */
-  thunar_vfs_path_list_free (location_button->drop_path_list);
+  thunar_g_file_list_free (location_button->drop_file_list);
 
   /* be sure to cancel any pending enter timeout */
   if (G_UNLIKELY (location_button->enter_timeout_id != 0))
@@ -425,7 +398,7 @@ static GdkDragAction
 thunar_location_button_get_dest_actions (ThunarLocationButton *location_button,
                                          GdkDragContext       *context,
                                          GtkWidget            *button,
-                                         guint                 time)
+                                         guint                 timestamp)
 {
   GdkDragAction actions = 0;
   GdkDragAction action = 0;
@@ -434,11 +407,11 @@ thunar_location_button_get_dest_actions (ThunarLocationButton *location_button,
   if (G_LIKELY (location_button->file != NULL))
     {
       /* determine the possible drop actions for the file (and the suggested action if any) */
-      actions = thunar_file_accepts_drop (location_button->file, location_button->drop_path_list, context, &action);
+      actions = thunar_file_accepts_drop (location_button->file, location_button->drop_file_list, context, &action);
     }
 
   /* tell Gdk whether we can drop here */
-  gdk_drag_status (context, action, time);
+  gdk_drag_status (context, action, timestamp);
 
   return actions;
 }
@@ -452,13 +425,12 @@ thunar_location_button_file_changed (ThunarLocationButton *location_button,
   ThunarIconFactory *icon_factory;
   GtkIconTheme      *icon_theme;
   GtkSettings       *settings;
-#if GTK_CHECK_VERSION(2,8,0)
-  const gchar       *icon_name;
-#endif
   GdkPixbuf         *icon;
+  const gchar       *icon_name;
   gint               height;
   gint               width;
   gint               size;
+  const gchar       *custom_icon;
 
   _thunar_return_if_fail (THUNAR_IS_LOCATION_BUTTON (location_button));
   _thunar_return_if_fail (location_button->file == file);
@@ -467,17 +439,17 @@ thunar_location_button_file_changed (ThunarLocationButton *location_button,
   /* determine the icon theme for the widget */
   icon_theme = gtk_icon_theme_get_for_screen (gtk_widget_get_screen (GTK_WIDGET (location_button)));
 
-  /* the label is hidden for the file system root */
-  if (thunar_file_is_local (file) && thunar_file_is_root (file))
+  /* update and show the label widget (hide for the local root folder) */
+  if (thunar_file_is_local (file) && thunar_file_is_root (file)) 
     {
-      /* hide the label widget */
-      gtk_widget_hide (location_button->label);
+      /* hide the alignment in which the label would otherwise show up */
+      gtk_widget_hide (gtk_widget_get_parent (location_button->label));
     }
   else
     {
-      /* update and show the label widget */
+      /* set label to the file's display name and show the alignment (and thereby the label) */
       gtk_label_set_text (GTK_LABEL (location_button->label), thunar_file_get_display_name (file));
-      gtk_widget_show (location_button->label);
+      gtk_widget_show (gtk_widget_get_parent (location_button->label));
     }
 
   /* the image is only visible for certain special paths */
@@ -493,9 +465,12 @@ thunar_location_button_file_changed (ThunarLocationButton *location_button,
       /* update the icon for the image */
       icon_factory = thunar_icon_factory_get_for_icon_theme (icon_theme);
       icon = thunar_icon_factory_load_file_icon (icon_factory, file, location_button->file_icon_state, size);
-      gtk_image_set_from_pixbuf (GTK_IMAGE (location_button->image), icon);
+      if (G_LIKELY (icon != NULL))
+        {
+          gtk_image_set_from_pixbuf (GTK_IMAGE (location_button->image), icon);
+          g_object_unref (G_OBJECT (icon));
+        }
       g_object_unref (G_OBJECT (icon_factory));
-      g_object_unref (G_OBJECT (icon));
 
       /* show the image widget */
       gtk_widget_show (location_button->image);
@@ -506,13 +481,17 @@ thunar_location_button_file_changed (ThunarLocationButton *location_button,
       gtk_widget_hide (location_button->image);
     }
 
-#if GTK_CHECK_VERSION(2,8,0)
   /* setup the DnD icon for the button */
-  icon_name = thunar_file_get_custom_icon (file);
-  if (G_LIKELY (icon_name == NULL))
-    icon_name = thunar_file_get_icon_name (file, location_button->file_icon_state, icon_theme);
-  gtk_drag_source_set_icon_name (GTK_BIN (location_button)->child, icon_name);
-#endif
+  custom_icon = thunar_file_get_custom_icon (file);
+  if (custom_icon != NULL)
+    {
+      gtk_drag_source_set_icon_name (GTK_BIN (location_button)->child, custom_icon);
+    }
+  else
+    {
+      icon_name = thunar_file_get_icon_name (file, location_button->file_icon_state, icon_theme);
+      gtk_drag_source_set_icon_name (GTK_BIN (location_button)->child, icon_name);
+    }
 }
 
 
@@ -596,6 +575,8 @@ thunar_location_button_button_release_event (GtkWidget            *button,
                                              ThunarLocationButton *location_button)
 {
   ThunarApplication *application;
+  ThunarPreferences *preferences;
+  gboolean           open_in_tab;
 
   _thunar_return_val_if_fail (THUNAR_IS_LOCATION_BUTTON (location_button), FALSE);
 
@@ -608,10 +589,22 @@ thunar_location_button_button_release_event (GtkWidget            *button,
       /* verify that we still have a valid file */
       if (G_LIKELY (location_button->file != NULL))
         {
-          /* open a new window for the folder */
-          application = thunar_application_get ();
-          thunar_application_open_window (application, location_button->file, gtk_widget_get_screen (GTK_WIDGET (location_button)));
-          g_object_unref (G_OBJECT (application));
+          preferences = thunar_preferences_get ();
+          g_object_get (preferences, "misc-middle-click-in-tab", &open_in_tab, NULL);
+          g_object_unref (preferences);
+
+          if (open_in_tab)
+            {
+              /* open in tab */
+              g_signal_emit (G_OBJECT (location_button), location_button_signals[CLICKED], 0, TRUE);
+            }
+          else
+            {
+              /* open a new window for the folder */
+              application = thunar_application_get ();
+              thunar_application_open_window (application, location_button->file, gtk_widget_get_screen (GTK_WIDGET (location_button)), NULL);
+              g_object_unref (G_OBJECT (application));
+            }
         }
     }
 
@@ -625,7 +618,7 @@ thunar_location_button_drag_drop (GtkWidget            *button,
                                   GdkDragContext       *context,
                                   gint                  x,
                                   gint                  y,
-                                  guint                 time,
+                                  guint                 timestamp,
                                   ThunarLocationButton *location_button)
 {
   GdkAtom target;
@@ -644,7 +637,7 @@ thunar_location_button_drag_drop (GtkWidget            *button,
   location_button->drop_occurred = TRUE;
 
   /* request the drag data from the source */
-  gtk_drag_get_data (button, context, target, time);
+  gtk_drag_get_data (button, context, target, timestamp);
 
   /* we'll call gtk_drag_finish() later */
   return TRUE;
@@ -657,11 +650,11 @@ thunar_location_button_drag_data_get (GtkWidget            *button,
                                       GdkDragContext       *context,
                                       GtkSelectionData     *selection_data,
                                       guint                 info,
-                                      guint                 time,
+                                      guint                 timestamp,
                                       ThunarLocationButton *location_button)
 {
-  gchar *uri_string;
-  GList  path_list;
+  gchar **uris;
+  GList   path_list;
 
   _thunar_return_if_fail (GTK_IS_WIDGET (button));
   _thunar_return_if_fail (THUNAR_IS_LOCATION_BUTTON (location_button));
@@ -670,14 +663,13 @@ thunar_location_button_drag_data_get (GtkWidget            *button,
   if (G_LIKELY (location_button->file != NULL))
     {
       /* transform the path into an uri list string */
-      path_list.data = thunar_file_get_path (location_button->file); path_list.next = path_list.prev = NULL;
-      uri_string = thunar_vfs_path_list_to_string (&path_list);
+      path_list.next = path_list.prev = NULL;
+      path_list.data = thunar_file_get_file (location_button->file);
 
       /* set the uri list for the drag selection */
-      gtk_selection_data_set (selection_data, selection_data->target, 8, (guchar *) uri_string, strlen (uri_string));
-
-      /* cleanup */
-      g_free (uri_string);
+      uris = thunar_g_file_list_to_stringv (&path_list);
+      gtk_selection_data_set_uris (selection_data, uris);
+      g_strfreev (uris);
     }
 }
 
@@ -690,7 +682,7 @@ thunar_location_button_drag_data_received (GtkWidget            *button,
                                            gint                  y,
                                            GtkSelectionData     *selection_data,
                                            guint                 info,
-                                           guint                 time,
+                                           guint                 timestamp,
                                            ThunarLocationButton *location_button)
 {
   GdkDragAction actions;
@@ -702,7 +694,7 @@ thunar_location_button_drag_data_received (GtkWidget            *button,
     {
       /* extract the URI list from the selection data (if valid) */
       if (selection_data->format == 8 && selection_data->length > 0)
-        location_button->drop_path_list = thunar_vfs_path_list_from_string ((const gchar *) selection_data->data, NULL);
+        location_button->drop_file_list = thunar_g_file_list_new_from_string ((const gchar *) selection_data->data);
 
       /* reset the state */
       location_button->drop_data_ready = TRUE;
@@ -715,24 +707,24 @@ thunar_location_button_drag_data_received (GtkWidget            *button,
       location_button->drop_occurred = FALSE;
 
       /* determine the drop dest actions */
-      actions = thunar_location_button_get_dest_actions (location_button, context, button, time);
+      actions = thunar_location_button_get_dest_actions (location_button, context, button, timestamp);
       if (G_LIKELY ((actions & (GDK_ACTION_COPY | GDK_ACTION_MOVE | GDK_ACTION_LINK)) != 0))
         {
           /* as the user what to do with the drop data */
           action = (context->action == GDK_ACTION_ASK)
-                 ? thunar_dnd_ask (button, location_button->file, location_button->drop_path_list, time, actions)
+                 ? thunar_dnd_ask (button, location_button->file, location_button->drop_file_list, timestamp, actions)
                  : context->action;
 
           /* perform the requested action */
           if (G_LIKELY (action != 0))
-            succeed = thunar_dnd_perform (button, location_button->file, location_button->drop_path_list, action, NULL);
+            succeed = thunar_dnd_perform (button, location_button->file, location_button->drop_file_list, action, NULL);
         }
 
       /* tell the peer that we handled the drop */
-      gtk_drag_finish (context, succeed, FALSE, time);
+      gtk_drag_finish (context, succeed, FALSE, timestamp);
 
       /* disable the highlighting and release the drag data */
-      thunar_location_button_drag_leave (button, context, time, location_button);
+      thunar_location_button_drag_leave (button, context, timestamp, location_button);
     }
 }
 
@@ -741,7 +733,7 @@ thunar_location_button_drag_data_received (GtkWidget            *button,
 static void
 thunar_location_button_drag_leave (GtkWidget            *button,
                                    GdkDragContext       *context,
-                                   guint                 time,
+                                   guint                 timestamp,
                                    ThunarLocationButton *location_button)
 {
   _thunar_return_if_fail (GTK_IS_BUTTON (button));
@@ -761,9 +753,9 @@ thunar_location_button_drag_leave (GtkWidget            *button,
   /* reset the "drop data ready" status and free the path list */
   if (G_LIKELY (location_button->drop_data_ready))
     {
-      thunar_vfs_path_list_free (location_button->drop_path_list);
+      thunar_g_file_list_free (location_button->drop_file_list);
       location_button->drop_data_ready = FALSE;
-      location_button->drop_path_list = NULL;
+      location_button->drop_file_list = NULL;
     }
 
   /* be sure to cancel any running enter timeout */
@@ -778,7 +770,7 @@ thunar_location_button_drag_motion (GtkWidget            *button,
                                     GdkDragContext       *context,
                                     gint                  x,
                                     gint                  y,
-                                    guint                 time,
+                                    guint                 timestamp,
                                     ThunarLocationButton *location_button)
 {
   ThunarFileIconState file_icon_state;
@@ -810,16 +802,16 @@ thunar_location_button_drag_motion (GtkWidget            *button,
       if (G_LIKELY (target == gdk_atom_intern_static_string ("text/uri-list")))
         {
           /* request the drop data from the source */
-          gtk_drag_get_data (button, context, target, time);
+          gtk_drag_get_data (button, context, target, timestamp);
         }
 
       /* tell GDK that it cannot drop here (yet!) */
-      gdk_drag_status (context, 0, time);
+      gdk_drag_status (context, 0, timestamp);
     }
   else
     {
       /* check whether we can drop into the buttons' folder */
-      actions = thunar_location_button_get_dest_actions (location_button, context, button, time);
+      actions = thunar_location_button_get_dest_actions (location_button, context, button, timestamp);
 
       /* determine the new file icon state */
       file_icon_state = (actions == 0) ? THUNAR_FILE_ICON_STATE_DEFAULT : THUNAR_FILE_ICON_STATE_DROP;
@@ -994,7 +986,7 @@ thunar_location_button_set_file (ThunarLocationButton *location_button,
       g_signal_connect_swapped (G_OBJECT (file), "destroy", G_CALLBACK (thunar_location_button_file_destroy), location_button);
 
       /* update our internal state for the new file (if realized) */
-      if (GTK_WIDGET_REALIZED (location_button))
+      if (gtk_widget_get_realized (GTK_WIDGET (location_button)))
         thunar_location_button_file_changed (location_button, file);
     }
 
@@ -1014,7 +1006,7 @@ void
 thunar_location_button_clicked (ThunarLocationButton *location_button)
 {
   _thunar_return_if_fail (THUNAR_IS_LOCATION_BUTTON (location_button));
-  g_signal_emit (G_OBJECT (location_button), location_button_signals[CLICKED], 0);
+  g_signal_emit (G_OBJECT (location_button), location_button_signals[CLICKED], 0, FALSE);
 }
 
 
